﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading; 

using Arction.WPF.LightningChartUltimate;
using Arction.WPF.LightningChartUltimate.Views;
using Arction.WPF.LightningChartUltimate.Views.ViewXY;
using Arction.WPF.LightningChartUltimate.Axes;
using Arction.WPF.LightningChartUltimate.SeriesXY; 

//Normally the scrolling view scrolls from right to left.
//This example shows how to use optimized scrolling view, from left to right, 
//by using a reversed axis and data point array reversing.
//Labels are manipulated by event handler in reversed mode.  

namespace WpfDataScrollingBackwards
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        LightningChartUltimate m_chartAllData;
        LightningChartUltimate m_chartScrolling;

        SeriesPoint[] m_points;
        DispatcherTimer m_timerDataFeeded;
        bool m_forward = true;
        int m_iLastIndexSetToFwd = -1;
        int m_iLastIndexSetToBackwards = -1;
        const int PointCountPerRound = 1; 

        public MainWindow()
        {
            InitializeComponent();

            m_points = CreateDataSet(); 

            CreateAllDataChart();

            CreateScrollingDataChart();

            m_timerDataFeeded = new DispatcherTimer();
            m_timerDataFeeded.Interval = TimeSpan.FromMilliseconds(10);
            m_timerDataFeeded.Tick += new EventHandler(m_timerDataFeeded_Tick);
            m_timerDataFeeded.Start(); 
        }

       
        //Add data charts shows the whole data set 
        void CreateAllDataChart()
        {

            m_chartAllData = new LightningChartUltimate();
            gridMain.Children.Add(m_chartAllData);
            Grid.SetRow(m_chartAllData, 1);

            SeriesPoint[] points = CreateDataSet(); 

            m_chartAllData.BeginUpdate();
            m_chartAllData.Title.Text = "All data"; 

            ViewXY v = m_chartAllData.ViewXY;
            AxisY yAxis = v.YAxes[0]; 
            AxisX xAxis = v.XAxes[0];

            PointLineSeries pls = new PointLineSeries(v, xAxis, yAxis);
            pls.LineStyle.Width = 1f;
            pls.LineStyle.AntiAliasing = LineAntialias.None;
            pls.AddPoints(points, false); 
            pls.MouseInteraction = false; 
            v.PointLineSeries.Add(pls);

            xAxis.ValueType = AxisValueType.Number;
            xAxis.SetRange(0, points[points.Length - 1].X);
            yAxis.SetRange(-100, 1000);

            v.LegendBox.Visible = false; 

            m_chartAllData.EndUpdate(); 

        }


        //Scrolling data chart scrolls through all data, using a zoomed-in view.  
        void CreateScrollingDataChart()
        {

            m_chartScrolling = new LightningChartUltimate();
            gridMain.Children.Add(m_chartScrolling);
            Grid.SetRow(m_chartScrolling, 2);

            

            m_chartScrolling.BeginUpdate();
            m_chartScrolling.Title.Text = "Scrolling chart - left, right"; 

            ViewXY v = m_chartScrolling.ViewXY;
            v.DropOldSeriesData = true; 

            AxisY yAxis = v.YAxes[0];
            AxisX xAxis = v.XAxes[0];

            PointLineSeries pls = new PointLineSeries(v, xAxis, yAxis);
            pls.LineStyle.Width = 1f;
            pls.LineStyle.AntiAliasing = LineAntialias.None;
            pls.MouseInteraction = false;
            pls.ScrollingStabilizing = true; 
            
            v.PointLineSeries.Add(pls);


            yAxis.SetRange(-100, 1000);
            
            xAxis.ValueType = AxisValueType.Number;
            xAxis.ScrollMode = XAxisScrollMode.Scrolling;
            xAxis.AutoDivSpacing = false;
            xAxis.MajorDiv = 1;
            xAxis.KeepDivCountOnRangeChange = false;
            xAxis.SetRange(0, 20);
            xAxis.AutoFormatLabels = false;
            xAxis.FormatValueLabel += new AxisBase.FormatValueLabelHandler(xAxis_FormatValueLabel);
            
            v.LegendBox.Visible = false; 

            m_chartScrolling.EndUpdate();

        }

        string xAxis_FormatValueLabel(AxisBase axis, double value)
        {
            if(m_forward)
                return value.ToString("0");   
            else
                return (-value).ToString("0");   
        }

        void m_timerDataFeeded_Tick(object sender, EventArgs e)
        {
            //Feed one data point to the scrolling chart. 

            LightningChartUltimate chart =  m_chartScrolling;
            chart.BeginUpdate();
            AxisX xAxis = chart.ViewXY.XAxes[0];
            PointLineSeries pls = chart.ViewXY.PointLineSeries[0]; 

            if (m_forward)
            {
                //Forwards scrolling 
                if (xAxis.Reversed)
                {
                    ChangeFromBackwardsModeToForwardMode(); 
                }

                int iPointCountToAdd = PointCountPerRound;
                int iFirstIndex = m_iLastIndexSetToFwd+1; 
                SeriesPoint[] points = GetPointsFwd(iFirstIndex, iPointCountToAdd);
                if (points.Length > 0)
                {
                    pls.AddPoints(points, false);
                    m_chartScrolling.ViewXY.XAxes[0].ScrollPosition = points[points.Length - 1].X; 
                    m_iLastIndexSetToFwd += points.Length;
                } 
            }
            else
            {
                //Backwards scrolling
                if (!xAxis.Reversed)
                {
                    ChangeFromForwardModeToBackwardsMode();
                }

                int iPointCountToAdd = PointCountPerRound;
                int iFirstIndex = m_iLastIndexSetToBackwards - 1;
                SeriesPoint[] points = GetPointsBackwards(iFirstIndex - iPointCountToAdd, iPointCountToAdd);
                if (points.Length > 0)
                {
                    pls.AddPoints(points, false);
                    m_chartScrolling.ViewXY.XAxes[0].ScrollPosition = points[points.Length - 1].X;
                    m_iLastIndexSetToBackwards -= points.Length;
                } 
            }

            chart.EndUpdate(); 
        }

        private void ChangeFromForwardModeToBackwardsMode()
        {
            AxisX xAxis =  m_chartScrolling.ViewXY.XAxes[0];
            double xMin = xAxis.Minimum;
            double xMax = xAxis.Maximum;
            
            xAxis.Reversed = true;
            xAxis.SetRange(-xMax, -xMin);

            //Seek the indices of points that are first and last in the current range 
            int iFirst, iLast; 
            SeekFirstAndLastPointIndicesFromRange(xMin,xMax, out iFirst, out iLast);

            PointLineSeries pls = m_chartScrolling.ViewXY.PointLineSeries[0];
            pls.Clear();
            SeriesPoint[] points = GetPointsBackwards(iFirst, iLast-iFirst);
            pls.AddPoints(points, false);

            m_iLastIndexSetToBackwards = iFirst;
            
            xAxis.ScrollPosition = points[points.Length - 1].X;

        }

        private void ChangeFromBackwardsModeToForwardMode()
        {
            AxisX xAxis = m_chartScrolling.ViewXY.XAxes[0];
            xAxis.Reversed = false;
            double xMin = -xAxis.Maximum;
            double xMax = -xAxis.Minimum;
            xAxis.SetRange(xMin, xMax);

            //Seek the indices of points that are first and last in the current range 
            int iFirst, iLast;
            SeekFirstAndLastPointIndicesFromRange(xMin, xMax, out iFirst, out iLast);

            PointLineSeries pls = m_chartScrolling.ViewXY.PointLineSeries[0];
            pls.Clear();
            SeriesPoint[] points = GetPointsFwd(iFirst, iLast - iFirst);
            pls.AddPoints(points, false);

            m_iLastIndexSetToFwd = iLast;
            
            xAxis.ScrollPosition = points[points.Length - 1].X;
        }

        void SeekFirstAndLastPointIndicesFromRange(double xMin, double xMax, out int firstIndex, out int lastIndex)
        {
            int iPointsLen = m_points.Length; 
            firstIndex = 0; 
            lastIndex = iPointsLen-1;

            //Seek first
            for (int i = 0; i < iPointsLen; i++)
            {
                if (m_points[i].X < xMin)
                    firstIndex = i;
                else
                    break; 
            }

            //Seek last 
            for (int i = iPointsLen - 1; i >= 0; i--)
            {
                if (m_points[i].X > xMax)
                    lastIndex = i;
                else
                    break; 
            }

        }


        SeriesPoint[] GetPointsFwd(int startIndex, int pointCount)
        {
            int iPointCountToGet = pointCount;

            if (m_points.Length < startIndex + pointCount)
                iPointCountToGet = m_points.Length - startIndex;

            SeriesPoint[] points = new SeriesPoint[iPointCountToGet];
            Array.Copy(m_points, startIndex, points, 0, iPointCountToGet);
            return points; 
        }


        SeriesPoint[] GetPointsBackwards(int startIndex, int pointCount)
        {
            int iPointCountToGet = pointCount;

            if (m_points.Length < startIndex + pointCount)
                iPointCountToGet = m_points.Length - startIndex;

            SeriesPoint[] points = new SeriesPoint[iPointCountToGet];

            if (startIndex + iPointCountToGet <= 0)
            {
                iPointCountToGet += startIndex; 
                startIndex = 0;
            }
            for (int i = 0; i < iPointCountToGet; i++)
            {
                points[i].Y = m_points[startIndex + iPointCountToGet - i].Y;
                points[i].X = -m_points[startIndex + iPointCountToGet - i].X; 
            }
          
            return points;
        }

        
        
        SeriesPoint[] CreateDataSet()
        {
            double xInterval = 0.1;

            int pointCount =  10001;
            SeriesPoint[] points = new SeriesPoint[pointCount];
            double x,y; 

            for (int i = 0; i < pointCount; i++)
            {
                x = (double)i * xInterval;

                y = 0; 
                if (x % 1.0 == 0)
                {
                    y = (double)i/10.0; 
                }

                points[i].X = x;
                points[i].Y = y; 
            }
            return points;
        }


        private void radioButton_Checked(object sender, RoutedEventArgs e)
        {
            if (m_timerDataFeeded != null)
            {

                if (radioButtonForward.IsChecked == true)
                {
                    if (m_timerDataFeeded.IsEnabled == false)
                        m_timerDataFeeded.Start();
                    m_forward = true;
                }
                else if (radioButtonBackwards.IsChecked == true)
                {
                    if (m_timerDataFeeded.IsEnabled == false)
                        m_timerDataFeeded.Start();
                    m_forward = false;
                }
                else //Stop 
                {
                    if (m_timerDataFeeded.IsEnabled == true)
                        m_timerDataFeeded.Stop();

                }
            }
        }
    }

}
